<?php

//--------------------------------------------------

namespace Bbg\ProductAssets;

//--------------------------------------------------

use Bbg\ProductAssets\Assets\BundleAssets;
use Bbg\ProductAssets\Assets\CustomizableAssets;
use Bbg\ProductAssets\Assets\ProductAssets;
use Bbg\ProductAssets\Cache\AbstractCache;
use Bbg\ProductAssets\Exceptions\HttpClientExceptions;
use Exception;
use stdClass;

//--------------------------------------------------

class Client
{

//--------------------------------------------------
// Public:Propertys


//--------------------------------------------------
// Private:Propertys
    const ENV_SANDBOX = "SANDBOX";

//--------------------------------------------------
// Protected:Propertys


//--------------------------------------------------
// Constants
    const ENV_PRODUCTION = "PRODUCTION";
    const ENVS = array(
        self::ENV_SANDBOX => "https://assets-ecom-products-dev.fra1.digitaloceanspaces.com",
        self::ENV_PRODUCTION => "https://assets-ecom-products-pro.fra1.digitaloceanspaces.com",
    );
    const TYPE_PRODUCT = "PRODUCT";
    const TYPE_BUNDLE = "BUNDLE";
    const TYPE_CUSTOMIZABLE = "CUSTOMIZABLE";
    const TYPES = array(
        self::TYPE_PRODUCT => "ProductAssets",
        self::TYPE_BUNDLE => "BundleAssets",
        self::TYPE_CUSTOMIZABLE => "CustomizableAssets",
    );
    static private $cache = null;

//--------------------------------------------------

    /**
     * No Information Found
     * @since 1.0
     */
    private function __construct()
    {

    }

//--------------------------------------------------

    public static function getProduct(string $sku, ?string $environment = null): ProductAssets
    {
        return new ProductAssets(
            $sku,
            self::loadJSONFile(ProductAssets::getJSONPath($sku), $environment)
        );
    }

//--------------------------------------------------

    public static function loadJSONFile(string $path, ?string $environment = null): stdClass
    {
        $environment = (isset(self::ENVS[$environment]) ? $environment : self::ENV_SANDBOX);
        $cacheHandler = self::getCache();
        $httpClient = new HttpClient();

        if ((is_object($cacheHandler)) && ($cacheHandler instanceof AbstractCache)) {
            if ($cacheHandler->exists($path, $environment)) {
                return $cacheHandler->read($path, $environment);
            }
        }

        $jsonData = $httpClient->request("GET", self::ENVS[$environment] . $path);

        if ($httpClient->getError() > 0) {
            throw new HttpClientExceptions("curl error", $httpClient->getError());
        }

        if ($httpClient->getStatus() !== 200) {
            throw new HttpClientExceptions("http status error", $httpClient->getStatus());
        }

        try {
            $jsonData = json_decode($jsonData);
        } catch (Exception $e) {
            throw new HttpClientExceptions("invalid json data", 0, $e);
        }

        if ((is_object($cacheHandler)) && ($cacheHandler instanceof AbstractCache)) {
            $cacheHandler->write($path, $jsonData, $environment);
        }

        return $jsonData;
    }

//--------------------------------------------------

    public static function getCache(): ?AbstractCache
    {
        return self::$cache;
    }

//--------------------------------------------------

    public static function setCache(AbstractCache $cache)
    {
        self::$cache = $cache;
    }

//--------------------------------------------------

    public static function getBundle(string $sku, ?string $environment = null): BundleAssets
    {
        return new BundleAssets(
            $sku,
            self::loadJSONFile(ProductAssets::getJSONPath($sku), $environment)
        );
    }

//--------------------------------------------------

    public static function getCustomizable(string $sku, ?string $environment = null): CustomizableAssets
    {
        return new CustomizableAssets(
            $sku,
            self::loadJSONFile(ProductAssets::getJSONPath($sku), $environment)
        );
    }

//--------------------------------------------------

    /**
     * No Information Found
     * @since 1.0
     */
    private function __clone()
    {

    }

//--------------------------------------------------

}

//--------------------------------------------------

?>